@@ -13,8 +13,10 @@ import org.gcube.portlets.user.tdtemplate.client.event.TemplateComplitedEvent;
 import org.gcube.portlets.user.tdtemplate.client.resources.TdTemplateAbstractResources;
 import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDefinition;
 import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDialogManager;
+import org.gcube.portlets.user.tdtemplate.client.template.flow.WindowFlowCreate;
 import org.gcube.portlets.user.tdtemplate.client.template.view.suggestion.ConstraintSuggestionLabel;
 import org.gcube.portlets.user.tdtemplate.client.template.view.suggestion.SuggestionContainer;
+import org.gcube.portlets.user.tdtemplate.shared.TdFlowModel;
 
 import com.extjs.gxt.ui.client.Style.ButtonScale;
 import com.extjs.gxt.ui.client.Style.IconAlign;
@@ -24,13 +26,17 @@ import com.extjs.gxt.ui.client.event.BaseEvent;
 import com.extjs.gxt.ui.client.event.ButtonEvent;
 import com.extjs.gxt.ui.client.event.Events;
 import com.extjs.gxt.ui.client.event.Listener;
+import com.extjs.gxt.ui.client.event.MenuEvent;
 import com.extjs.gxt.ui.client.event.SelectionListener;
 import com.extjs.gxt.ui.client.util.Margins;
 import com.extjs.gxt.ui.client.widget.ContentPanel;
 import com.extjs.gxt.ui.client.widget.LayoutContainer;
 import com.extjs.gxt.ui.client.widget.button.Button;
+import com.extjs.gxt.ui.client.widget.button.SplitButton;
 import com.extjs.gxt.ui.client.widget.layout.BorderLayout;
 import com.extjs.gxt.ui.client.widget.layout.BorderLayoutData;
+import com.extjs.gxt.ui.client.widget.menu.Menu;
+import com.extjs.gxt.ui.client.widget.menu.MenuItem;
 import com.extjs.gxt.ui.client.widget.menu.SeparatorMenuItem;
 import com.extjs.gxt.ui.client.widget.toolbar.FillToolItem;
 import com.extjs.gxt.ui.client.widget.toolbar.ToolBar;
@@ -50,6 +56,14 @@ import com.google.gwt.user.client.ui.Widget;
  */
 public class TemplatePanel {
 	
+	/**
+	 * 
+	 */
+	private static final String EDIT_FLOW = "Edit Flow";
+	/**
+	 * 
+	 */
+	private static final String CREATE_FLOW = "Create Flow";
 	protected FlexTable flexTableTemplate = new FlexTable();
 	protected LayoutContainer flexTableContainer = new LayoutContainer();
 	private int numColumns;
@@ -69,6 +83,10 @@ public class TemplatePanel {
 	private BorderLayout borderLayout;
 	private BorderLayoutData centerData;
 	private BorderLayoutData southData;
+	private Button addFlow;
+	private MenuItem createFlow;
+	private MenuItem removeFlow;
+	private SelectionListener listenerAddFlow;
 	
 	
 	/**
@@ -297,9 +315,99 @@ public class TemplatePanel {
 		
 		toolbar.add(new FillToolItem());
 		
-//		toolbar.add(filterBySelection);
+		addFlow = new SplitButton("Flow");
+		addFlow.setIcon(TdTemplateAbstractResources.postOperation24());
+		
+		addFlow.setScale(ButtonScale.MEDIUM);
+		addFlow.setIconAlign(IconAlign.TOP);
+//		addColumnButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
+		addFlow.setMenu(createFlowMenuAndListener());
+		
+		listenerAddFlow = new SelectionListener<ButtonEvent>() {
+			
+			@Override
+			public void componentSelected(ButtonEvent ce) {
+				addFlow.getMenu().show(addFlow);
+				getFlow();
+			}
+		};
+		
+		addFlow.addSelectionListener(listenerAddFlow);
+		
+		enablePostOperation(false);
+		
+		toolbar.add(addFlow);
 		tableContainer.setTopComponent(toolbar);
 	}
+	
+	private Menu createFlowMenuAndListener() {
+		  
+	    Menu menu = new Menu();
+	    
+	    createFlow = new MenuItem(CREATE_FLOW);
+	    createFlow.setIcon(TdTemplateAbstractResources.pencil());
+	    createFlow.addSelectionListener(new SelectionListener<MenuEvent>() {
+
+			@Override
+			public void componentSelected(MenuEvent ce) {
+				WindowFlowCreate.geInstance().show();
+			}
+	    });
+	    
+		WindowFlowCreate.geInstance().addListener(Events.Hide, new Listener<BaseEvent>() {
+
+			@Override
+			public void handleEvent(BaseEvent be) {
+				
+				GWT.log("Hide Create/Update flow");
+				
+				boolean flowCreated = WindowFlowCreate.geInstance().flowExists();
+				
+				GWT.log("flowCreated: "+flowCreated);
+				
+				if(flowCreated){
+					createFlow.setText(EDIT_FLOW);
+					addFlow.setIcon(TdTemplateAbstractResources.postOperation24Ok());
+					BaloonPanel baloon = new BaloonPanel("Flow attached!", true);
+
+					int zIndex = controller.getWindowZIndex();
+					int zi = zIndex+1;
+					baloon.getElement().getStyle().setZIndex(zi);
+					baloon.showRelativeTo(addFlow);
+				}
+				else{
+					createFlow.setText(CREATE_FLOW);
+					addFlow.setIcon(TdTemplateAbstractResources.postOperation24());
+				}
+				
+				removeFlow.setVisible(flowCreated);
+			}
+		});
+	    
+	    removeFlow = new MenuItem("Remove Flow");
+	    removeFlow.setIcon(TdTemplateAbstractResources.close());
+	    removeFlow.addSelectionListener(new SelectionListener<MenuEvent>() {
+
+			@Override
+			public void componentSelected(MenuEvent ce) {
+				WindowFlowCreate.geInstance().resetFlowCreatePanel();
+				removeFlow.setVisible(false);
+				createFlow.setText(CREATE_FLOW);
+				addFlow.setIcon(TdTemplateAbstractResources.postOperation24());
+			}
+	    });
+	    
+	    removeFlow.setVisible(false);
+	    
+	    menu.add(createFlow);
+	    menu.add(removeFlow);
+	 
+	    return menu;  
+	}  
+	
+	public void enablePostOperation(boolean bool){
+		addFlow.setEnabled(bool);
+	}
 
 	public void enableFilter(boolean bool){
 //		filterBySelection.setEnabled(bool);
@@ -310,8 +418,6 @@ public class TemplatePanel {
 			borderLayout.expand(LayoutRegion.SOUTH);
 		
 		southContainer.setEnabled(bool);
-		
-	
 		/*
 		if(!bool)
 			borderLayout.collapse(LayoutRegion.SOUTH);
@@ -509,6 +615,7 @@ public class TemplatePanel {
 				isValid = false;
 				refreshSuggestion(TdTemplateConstants.SUGGESTION, TdTemplateConstants.PLEASE_SET_TYPE_TO_COLUMN_NUMBER+(col.getColumnIndex()+1));
 				TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(false));
+				enablePostOperation(false);
 //				enableFilter(false);
 //				resetFilterPanel();
 				break;
@@ -518,6 +625,7 @@ public class TemplatePanel {
 		if(isValid){
 			refreshSuggestion(TdTemplateConstants.TEMPLATE_COMPLETED, TdTemplateConstants.NOW_IS_POSSIBLE_TO_SAVE_THE_TEMPLATE_CREATED, TdTemplateAbstractResources.handsUP());
 			TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(true));
+			enablePostOperation(true);
 			/*enableFilter(true);
 			enanbleColumnTypes(false);
 			BaloonPanel baloonPanel = new BaloonPanel("Do you want add filters?", true);
@@ -529,4 +637,47 @@ public class TemplatePanel {
 //			setCheckColumnsEnabled(true);
 		}
 	}
+
+
+	/**
+	 * @return flow is FLOW is created or read only, otherwise null
+	 * 
+	 */
+	public TdFlowModel getFlow() {
+
+		if(WindowFlowCreate.geInstance().flowExists() || WindowFlowCreate.geInstance().flowIsReadOnly())
+			return WindowFlowCreate.geInstance().getFlow();
+
+		return null;
+	}
+	
+	public void setFlowAsReadOnly(boolean bool){
+		WindowFlowCreate.geInstance().setFlowAsReadOnly(bool);
+		
+		if(bool){
+			addFlow.setEnabled(true);
+			removeFlow.setVisible(false);
+			createFlow.setText("View Flow");
+			createFlow.setIcon(TdTemplateAbstractResources.view());
+			createFlow.setVisible(true);
+			
+			addFlow.removeSelectionListener(listenerAddFlow);
+			
+			addFlow.addSelectionListener(new SelectionListener<ButtonEvent>() {
+
+				@Override
+				public void componentSelected(ButtonEvent ce) {
+					WindowFlowCreate.geInstance().show();
+					
+				}
+			});
+		}
+	}
+	
+	/**
+	 * @param bool
+	 */
+	public void setAddFlowAsVisible(boolean bool) {
+		addFlow.setVisible(bool);
+	}
 }
